/*
 * oled_v1.h
 *	Driver for OLED modules with WS0010 and RS0010 drivers and with capability for SPI
 *	works with ws0010.h and ws0010.c (where is physical layer - bit banding SPI)
 *
 *  Created on: 2. 1. 2020
 *      Author: misa
 */

#ifndef OLED_V1_H_
#define OLED_V1_H_

#include "ws0010.h"

// OLED driver registers
#define OLED_CLEAR					0x01

#define OLED_HOME					0x02

#define OLED_ENTRY_MODE				0x04
	#define OLED_SHIFT_DISPLAY	1
	#define OLED_CURSOR_DECREMENT		0
	#define OLED_CURSOR_INCREMENT		2

#define OLED_ONOFF			0x08
	#define OLED_OFF			0
	#define OLED_ON			4
	#define OLED_CURSOR_OFF			0
	#define OLED_CURSOR_ON			2
	#define OLED_CURSOR_NOBLINK		0
	#define OLED_CURSOR_BLINK		1

#define OLED_CURSOR_SHIFT	0x10
	#define OLED_SHIFT_CURSOR_LEFT				0<<2
	#define OLED_SHIFT_CURSOR_RIGHT			1<<2
	#define OLED_SHIFT_DISPLAY_LEFT			2<<2
	#define OLED_SHIFT_DISPLAY_RIGHT		3<<2

#define OLED_GRAPHIC_PWR	    0x10 | 0b11
	#define OLED_GRAPHIC_MODE		0b1<<3
	#define OLED_TEXT_MODE			0
	#define OLED_PWR_ON				0b1<<2
	#define OLED_PWR_OFF				0

#define OLED_FUNCTION_SET			0x20
	#define OLED_FONT5x7				0
	#define OLED_FONT5x10			4
	#define OLED_ONE_LINE			0
	#define OLED_TWO_LINE			8
	#define OLED_COM_4BIT			0
	#define OLED_COM_8BIT			16
	#define OLED_FONT_TABLE_ENJA			0
	#define OLED_FONT_TABLE_WEU			1
	#define OLED_FONT_TABLE_ENRU			2
	#define OLED_FONT_TABLE_WEU2			3

#define OLED_CGRAM_SET				0x40

#define OLED_DDRAM_SET				0x80

// OLED functions
uint8_t oled_init(uint8_t font_table, uint8_t mode);
uint8_t oled_command(uint8_t command);
uint8_t oled_data(uint8_t data);
uint8_t oled_puts(uint8_t * text);
uint8_t oled_store_characters(uint8_t* array, uint8_t pos);

#define oled_gotoxy(X,Y)								oled_command(OLED_DDRAM_SET | (X + (0x40 * Y))) // set cursor on desired position, counts from 0, position 0,0 is left top
#define oled_putchar(CH)								oled_data(CH) // puts single character on display (at position of cursor)
#define oled_clear()												oled_command(OLED_CLEAR) // clear whole display (set into whole memory character 0x20)
#define oled_home()												 oled_command(OLED_HOME) // set cursor to 0,0 position, return display shift
#define oled_cursor_blink()  			oled_command(OLED_ONOFF | OLED_ON | OLED_CURSOR_ON | OLED_CURSOR_BLINK) // activate visible cursor blinking (underline)
#define oled_cursor_static()  		oled_command(OLED_ONOFF | OLED_ON | OLED_CURSOR_ON) // activate visible static cursor (underline)
#define oled_cursor_off()  					oled_command(OLED_ONOFF | OLED_ON) // deactivate visible cursor (underline)
#define oled_display_on()  					oled_command(OLED_ONOFF | OLED_ON) // enable display (does not change RAM content)
#define oled_display_off()  				oled_command(OLED_ONOFF) // disable display (does not change RAM content)
#define oled_cursor_right()					oled_command(OLED_CURSOR_SHIFT | OLED_SHIFT_CURSOR_RIGHT) // move cursor at higher address (to right)
#define oled_cursor_left()						oled_command(OLED_CURSOR_SHIFT | OLED_SHIFT_CURSOR_LEFT) // move cursor to lower address (to left)
#define oled_display_right()				oled_command(OLED_CURSOR_SHIFT | OLED_SHIFT_DISPLAY_RIGHT) // shift entire display to right
#define oled_display_left()					oled_command(OLED_CURSOR_SHIFT | OLED_SHIFT_DISPLAY_LEFT) // shift entire display to left
#define oled_read_address()					ws_get_status() // returns actual cursor position (uint8_t)
#define oled_pwr_off()										oled_command(OLED_GRAPHIC_PWR) // deselect Text/Graphic mode ! Consider call oled_init() after power on.


// function for graphic mode
#define oled_graphic_setx(XPOS) oled_command(OLED_DDRAM_SET | XPOS) // set x position (in pixels)
#define oled_graphic_sety(YPOS) oled_command(OLED_CGRAM_SET | YPOS) // set y position (in bytes / lines)
// for writing graphic data use ws_array_data() or slower single byte acces oled_data();

/*
 * Graphic mode data orientation
 * YPOS 		 XPOS ---------------->
 *  |   		 Byte  0  1  2  3  4 ...
 *  |   		 bit0  x  x  x  x  x ...
 *  |   		 bit1  x  x  x  x  x ...
 *  |   		 bit2  x  x  x  x  x ...
 *  |   		 bit3  x  x  x  x  x ...
 *  |   		 bit4  x  x  x  x  x ...
 *  |   		 bit5  x  x  x  x  x ...
 *  |   		 bit6  x  x  x  x  x ...
 *  V   		 bit7  x  x  x  x  x ...
 *
 */

#endif /* OLED_V1_H_ */
